@@ -3,7 +3,7 @@
 
 #
 # Created by Idel Fuschini 
-# Date: 01/01/10
+# Date: 01/08/10
 # Site: http://www.idelfuschini.it
 # Mail: idel.fuschini@gmail.com
 
@@ -33,7 +33,7 @@ package Apache2::AMFWURFLFilterMemcached;
 
   use vars qw($VERSION);
   my $CommonLib = new Apache2::AMFCommonLib ();
-  $VERSION= "3.08";
+  $VERSION= "3.09a";
   my %Capability;
   my %Array_fb;
   my %Array_id;
@@ -57,6 +57,7 @@ package Apache2::AMFWURFLFilterMemcached;
   $MobileArray{'lge'}='mobile';
   $MobileArray{'brew'}='mobile';
   $MobileArray{'webos'}='mobile';
+  $MobileArray{'kddi'}='mobile';
   $PCArray{'MSIE'}='msie';
   $PCArray{'MSIE 6'}='msie_6';
   $PCArray{'MSIE 7'}='msie_7';
@@ -230,7 +231,10 @@ sub loadConfigFile {
 
 			} else {
 				$CommonLib->printLog("The file is a xml file.");
-			    my $content = get ($downloadwurflurl);
+			        my $content = get ($downloadwurflurl);
+                                $content =~ s/\n//g;
+				$content =~ s/>/>\n/g;
+
 				my @rows = split(/\n/, $content);
 				my $row;
 				my $count=0;
@@ -274,11 +278,13 @@ sub loadConfigFile {
 		            $CommonLib->printLog("The size of document is: $document_length bytes");	       
 		        }
 				my $content = get ($patchwurflurl);
-				$CommonLib->printLog("Finish downloading  WURFL.xml");
+				$CommonLib->printLog("Finish downloading patch WURFL.xml");
 				if ($content eq "") {
 					$CommonLib->printLog("Couldn't get patch $patchwurflurl.");
 					ModPerl::Util::exit();
 				}
+				$content =~ s/\n//g;
+				$content =~ s/>/>\n/g;
 				my @rows = split(/\n/, $content);
 				my $row;
 				my $count=0;
@@ -290,11 +296,17 @@ sub loadConfigFile {
 				if (-e "$filePatch") {
 						$CommonLib->printLog("Start loading Web Patch File of WURFL");
 						if (open (IN,"$filePatch")) {
-							while (<IN>) {
-								 $r_id=parsePatchFile($_,$r_id);
-								 
-							}
+							my $filesize= -s $filePatch;
+							my $string_file;
+							read (IN,$string_file,$filesize);
 							close IN;
+							$string_file =~ s/\n//g;
+							$string_file =~ s/>/>\n/g;
+							my @arrayFile=split(/\n/, $string_file);
+							foreach my $line (@arrayFile) {
+								$r_id=parsePatchFile($line,$r_id);
+							}
+
 						} else {
 							$CommonLib->printLog("Error open file:$filePatch");
 							ModPerl::Util::exit();
@@ -380,7 +392,7 @@ sub IdentifyUAMethod {
   my $ua_toMatch;
   my $near_toFind=100;
   my $near_toMatch;
-  my %ArrayUAType=$CommonLib->GetMultipleUa($UserAgent);  
+  my %ArrayUAType=$CommonLib->GetMultipleUa(lc($UserAgent));  
   foreach $pair (reverse sort { $a <=> $b }  keys	 %ArrayUAType)
   {
       my $dummy=$ArrayUAType{$pair};
@@ -405,6 +417,7 @@ sub IdentifyPCUAMethod {
 		$id_find=$PCArray{$pair};
 	}
   }
+  if ($id_find) {} else {$id_find="";};
   if ($id_find eq "") { 
 	foreach $pair (%PatchArray_id)
 	{  
@@ -419,8 +432,15 @@ sub callparseWURFLFile {
 	 my ($output) = @_;
 	 my $r_id;
 	if (open (IN,"$output")) {
-		while (<IN>) {
-			$r_id=parseWURFLFile($_,$r_id);
+		my $filesize= -s $output;
+		my $string_file;
+		read (IN,$string_file,$filesize);
+		close IN;
+		$string_file =~ s/\n//g;
+		$string_file =~ s/>/>\n/g;
+		my @arrayFile=split(/\n/, $string_file);
+		foreach my $line (@arrayFile) {
+			$r_id=parseWURFLFile($line,$r_id);
 		}
 		close IN;			  
 	} else {
@@ -443,13 +463,13 @@ sub parseWURFLFile {
 		 } 
 	     if ($record =~ /\<device/o) {
 	        if (index($record,'user_agent') > 0 ) {
-	           $ua=substr($record,index($record,'user_agent') + 12,index($record,'"',index($record,'user_agent')+ 13)- index($record,'user_agent') - 12);
-			  if (index($ua,'BlackBerry') >0 ) {
-					$ua=substr($ua,index($ua,'BlackBerry'));
+	           $ua=lc(substr($record,index($record,'user_agent') + 12,index($record,'"',index($record,'user_agent')+ 13)- index($record,'user_agent') - 12));
+			  if (index($ua,'blackberry') >-1 ) {
+					$ua=substr($ua,index($ua,'blackberry'));
 			  }
 	        }	        
 	        if (index($record,'id') > 0 ) {
-	           $id=substr($record,index($record,'id') + 4,index($record,'"',index($record,'id')+ 5)- index($record,'id') - 4);	
+	           $id=substr($record,index($record,'id') + 4,index($record,'"',index($record,'id')+ 5)- index($record,'id') - 4);
 	        }	        
 	        if (index($record,'fall_back') > 0 ) {
 	           $fb=substr($record,index($record,'fall_back') + 11,index($record,'"',index($record,'fall_back')+ 12)- index($record,'fall_back') - 11);	           
@@ -480,8 +500,8 @@ sub parseWURFLFile {
 			   $Array_DDRcapability{"$val|$name"}=$value;
 			}
 		 }
-		 if ($record =~ /\<ver/o) {
-		     $WURFLVersion=$CommonLib->extValueTag("ver",$record);
+		 if ($record =~ /\/ver>/o) {
+		     $WURFLVersion=substr($record,index($record,'<ver>') + 5,index($record,"</ver>") - 9);
 		 }
 		 return $id;
 
@@ -501,10 +521,7 @@ sub parsePatchFile {
 		 } 
 	     if ($record =~ /\<device/o) {
 	        if (index($record,'user_agent') > 0 ) {
-	           $ua=substr($record,index($record,'user_agent') + 12,index($record,'"',index($record,'user_agent')+ 13)- index($record,'user_agent') - 12);
-			  if (index($ua,'BlackBerry') >0 ) {
-					$ua=substr($ua,index($ua,'BlackBerry'));
-			  }
+	           $ua=lc(substr($record,index($record,'user_agent') + 12,index($record,'"',index($record,'user_agent')+ 13)- index($record,'user_agent') - 12));
 	        }	        
 	        if (index($record,'id') > 0 ) {
 	           $id=substr($record,index($record,'id') + 4,index($record,'"',index($record,'id')+ 5)- index($record,'id') - 4);	
@@ -540,7 +557,7 @@ sub handler {
     my $f = shift;  
     my $capability2;
     my $variabile="";
-    my $user_agent=$f->headers_in->{'User-Agent'}|| '';
+    my $user_agent=lc($f->headers_in->{'User-Agent'}|| '');
     my $x_user_agent=$f->headers_in->{'X-Device-User-Agent'}|| '';
     my $x_operamini_phone_ua=$f->headers_in->{'X-OperaMini-Phone-Ua'}|| '';
     my $x_operamini_ua=$f->headers_in->{'X-OperaMini-Ua'}|| '';
@@ -563,10 +580,10 @@ sub handler {
     
 
     if ($x_user_agent) {
-       $user_agent=$x_user_agent;
+       $user_agent=lc($x_user_agent);
     }	  
     if ($x_operamini_phone_ua) {
-       $user_agent=$x_operamini_phone_ua;
+       $user_agent=lc($x_operamini_phone_ua);
     }
     if (($query_string) && $restmode eq 'true') {
     		  my @vars = split(/&/, $query_string); 	  
@@ -575,31 +592,33 @@ sub handler {
     						my ($v,$i) = split(/=/, $var);
     						$v =~ tr/+/ /;
     						$v =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
-    						$i =~ tr/+/ /;
-    						$i =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
-    						$i =~ s/<!--(.|\n)*-->//g;
+						if ($i) {
+							$i =~ tr/+/ /;
+							$i =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
+							$i =~ s/<!--(.|\n)*-->//g;
     						$ArrayQuery{$v}=$i;
+						}
     					}
     		  }
     	  if ($ArrayQuery{amf}) {
-    				$user_agent=$ArrayQuery{amf};
+    				$user_agent=lc($ArrayQuery{amf});
     	  }
 
     }    
 
-	if ($user_agent =~ m/Blackberry/i) {	 
-		$user_agent=substr($user_agent,index($user_agent,'BlackBerry'));
+	if ($user_agent =~ m/blackberry/i) {	 
+		$user_agent=substr($user_agent,index($user_agent,'blackberry'));
 		$mobile=1;
 	}
-	if ($user_agent =~ m/UP.link/i ) {
-		$user_agent=substr($user_agent,0,index($user_agent,'UP.Link') - 1);
+	if ($user_agent =~ m/up.link/i ) {
+		$user_agent=substr($user_agent,0,index($user_agent,'up.link') - 1);
 		$mobile=1;
 	}
     my $cookie = $f->headers_in->{Cookie} || '';
     $id=$CommonLib->readCookie($cookie);
+    $user_agent=lc($user_agent);
     if ($id eq ""){
                   if ($user_agent) {
-					my $lcuser_agent=lc($user_agent);
 	  			    if ($mobile==0) {
 						foreach my $pair (%MobileArray) {
 							if ($user_agent =~ m/$pair/i) {
@@ -610,9 +629,13 @@ sub handler {
 							$id=IdentifyPCUAMethod($user_agent);
 						}			            
 					}
+				        if ($id) {}else{$id="";};
 					if ($id eq "") { 
 							$id=IdentifyUAMethod($user_agent);
 					}
+					if ($id eq "") { 
+							$id='generic_web_browser';
+					}
 				  }	
      }                        
      if ($id ne "") {
@@ -648,17 +671,11 @@ sub handler {
 			if ($cookiecachesystem eq "true") {
 						$f->err_headers_out->set('Set-Cookie' => "amf=$id; path=/;");	
 			}		  			  
-	} else {
-	      	     #
-	      	     # unknown device 
-	      	     #
-				 if ($cookiecachesystem eq "true") {
-							$f->err_headers_out->set('Set-Cookie' => "amf=device_not_found; path=/;");	
-				  }		  			  
-	}
+	} 
     
 	$f->subprocess_env("AMF_VER" => $VERSION);
 	$f->subprocess_env("AMF_WURFLVER" => $WURFLVersion);
+	$f->headers_out->set("AMF-Ver"=> $VERSION);
 	if ($x_operamini_ua) {
 	    $f->subprocess_env("AMF_MOBILE_BROWSER" => $x_operamini_ua);
 	    $f->pnotes("mobile_browser" => $x_operamini_ua);
